class Rsg::Generators::Misc::MiscGenerator < Rsg::Generators::Base
  def banner
    say "Making some extra tweaks"
  end

  def remove_assets
    run "rm -rf app/assets"
  end

  def remove_gemfile_comments
    gsub_file "Gemfile", /^ *#\s*[^#\n]+\n/, ""
  end

  def remove_tzinfo_gem
    # Remove tzinfo-data since we don't have a need for it in dev / prod env, if
    # rsg becomes a thing on the platforms it's needed we can revisit
    gsub_file "Gemfile", /\ngem 'tzinfo-data'[^\n]+\n/, ""
  end

  def remove_gemfile_extra_whitespace
    gsub_file "Gemfile", /^( *gem[^\n]+)\n\n\n/, "\\1\n\n"
  end

  def isolate_rails_gem
    gsub_file "Gemfile", /^( *)(gem ["']rails["'][^\n]+)\n(?: *)(gem ['"]puma['"][^\n]+)\n$/, "\\1\\2\n\n\\3"
  end

  def disable_assets_generators
    return if api_mode?

    inject_into_file "config/application.rb", <<-CODE, before: /^\s*# Don't generate system test files./

    # Don't generate CSS files when using scaffolding commands, since we compile assets through webpacker
    config.generators.assets = false
    config.generators.scaffold_stylesheet = false
    config.generators.stylesheets = false

    # Most of time we can get away with a handful of methods in the default application_helper. Other ones
    # can be introduced as needed
    config.generators.helper = false
      CODE
  end

  def configure_pry
    append_gem "pry-rails", within_group: :development
  end

  # TODO: Skip puma on app generator, introduce a new Puma::InstallGenerator
  def puma_configs
    copy_file "puma.rb", "config/puma.rb", force: true
  end

  # TODO: Move to an i18n generator that does this and more
  def configure_environments
    # If by any chance we make use of translations, let them fail and get
    # caught early on the dev process
    uncomment_lines "config/environments/development.rb", /raise_on_missing_translations/
    uncomment_lines "config/environments/test.rb", /raise_on_missing_translations/
  end

  # TODO: Move to rsg-dox
  def healthcheck
    inject_into_file "config/routes.rb", <<-CODE, before: "end"

  # Defines a static healthcheck for probes. Please note that this won't
  # output any logs in production
  get :healthcheck, to: proc { [200, {}, ['OK']] }
    CODE
  end
end
